from django.http import (
    HttpResponse,
    HttpResponseRedirect,
)
from django.shortcuts import (
    get_object_or_404,
    render_to_response,
)
from django.views.generic import (
    RedirectView,
    TemplateView,
    View,
)
from django.core.exceptions import ValidationError
from url_shortener.models import (
    Bookmark,
    TooManyUrlsError,
    Url,
)
from utils import decode_in_alphabet
import settings
import json


class UrlRedirectionView(RedirectView):
    permanent = True

    def get_redirect_url(self, short_url):
        bookmark = Bookmark.get_bookmark(name=short_url)
        if bookmark is not None:
            return bookmark.url
        id = decode_in_alphabet(short_url, settings.URL_ALPHABET)
        url = get_object_or_404(Url, id=id)
        return url.long_url


class HomePageView(TemplateView):
    template_name = 'url_shortener/index.html'


class UrlCreationView(View):
    def post(self, request, **kwargs):
        try:
            url = Url.get_or_create_url(
                long_url=request.POST['long_url'],
                ip_address=request.META['REMOTE_ADDR']
            )
            host_name = request.META['HTTP_HOST']
            full_short_url = 'http://%s/%s' % (host_name, url.short_url)
            return HttpResponse(json.dumps({'short_url': full_short_url}))
        except KeyError:
            return HttpResponse('Unknown url!', status=400)
        except ValidationError:
            return HttpResponse('Not a valid url!', status=400)
        except TooManyUrlsError as e:
            return HttpResponse(e.message, status=400)
